!+
! Module: gti_mod.90
!
! Authors:  Peter Videler - SOC ESTEC <peter@videler.nl>
!          
! Description: stuff for Good Time Interval (GTI) creation
! 
! Comments: 
!
! $Id: gti_mod.f90,v 1.7 2001/07/06 15:32:33 cerd Exp $
!- 

module gti_mod

  use saslib
  use dal

  ! ......................................................
  implicit none 
  private 
  public createGTI
  ! ......................................................

contains

  ! --------------------------------------------------------------------------------
  subroutine initGTI(gti)
    ! --------------------------------------------------------------------------------
    ! init existing GTI table
!    use oal

    type (TableT), intent(inout)   :: gti
    type (ColumnT)                 :: col

!    real(double)                         :: starttime, stoptime
!    type (ExposureInfoType)              :: ExposureInfo  ! typedef from oal

    col = addColumn(gti, 'START', real64, 'sec')
    col = addColumn(gti, 'STOP' , real64, 'sec')

     ! need reference here
    call setAttribute(gti, "DATATYPE", "GTI.EL")
    call setAttribute(gti, "NUMGTI"  , 1)
    call setAttribute(gti, "HDUCLASS", "OGIP")
    call setAttribute(gti, "HDUCLAS1", "GTI" )
    call setAttribute(gti, "HDUCLAS2", "STANDARD")

    call setAttribute(gti, "TIMESYS", "TT", "XMM time is TT (Terrestial Time)")
    call setAttribute(gti, "MJDREF", 50814, "", "1998-01-01T00:00:00 (TT) expressed in MJD (TT)")
    call setAttribute(gti, "TIMEZERO", 0, "s", "Clock correction (if not zero)")
    call setAttribute(gti, "TIMEUNIT", "s", "All times in s unless specified otherwise")
    call setAttribute(gti, "CLOCKAPP", .true., "Clock correction applied")
    call setAttribute(gti, "TIMEREF", "LOCAL", "Reference location of photon arrival times")
    call setAttribute(gti, "TASSIGN", "SATELLITE", "Location of time assignment")

    call message(appMsg, verbose, "createGTI: Created empty GTI table")

!    call OAL_exposureInfo(exposureInfo)
!    starttime = ToTimeTag(exposureInfo%actualStartTime)
!    stoptime  = ToTimeTag(exposureInfo%actualEndTime)
!    call addGTI(tbl, starttime, stoptime)

  end subroutine initGTI


  ! --------------------------------------------------------------------------------
  subroutine addGTI(gti, fromtime, totime)
    ! --------------------------------------------------------------------------------
    ! add row with start and stop times to existing GTI table
    use dal

    type (TableT), intent(inout)   :: gti
    real(double) , intent(in)      :: fromtime, totime

    real(double), dimension(:), pointer :: fromt, tot
    integer ::  nrows
    type (columnT) :: col1, col2

    write(errstr,*)  "addGTI: Adding Good time interval from ", fromtime, ' to ', totime, ' (duration ', totime-fromtime, ' sec).'
    call message(appMsg, verbose)

    nrows = numberOfRows(gti)

    call insertRows(gti, nrows)    ! add one row

    col1 = column(gti, 'START') 
    col2 = column(gti, 'STOP') 
  
    ! fill last row with new values
    fromt => real64data(col1, nrows)
    tot   => real64data(col2, nrows)
    fromt(0) = fromtime
    tot  (0) = totime

    call release(col1)
    call release(col2)

  end subroutine addGTI


  ! --------------------------------------------------------------------------------
  function createGTI(set, good, time, exposureTime)  result(tbl)
    ! --------------------------------------------------------------------------------
    use dal
    use odfNames 

    type (DataSetT), intent(inout)         :: set
    logical,      dimension(:), intent(in) :: good
    real(double), dimension(:), intent(in) :: time  ! MID of exposure!!!
    real(single), dimension(:), intent(in) :: exposureTime

    type (tableT)                        :: tbl
    integer            :: i, istart, istop, sz
    logical            :: previous

    ! create empty GTI table according to OGIP standard
    tbl = addTable(set, name='STDGTI', numberOfRows=0)
    call initGTI(tbl)

    sz = size(good)
    if (sz /= size(time) .or. sz /= size(exposureTime)) &
      call error('createGTI', 'array sizes "time" "good" and "exposuretime" do not match')

    if (sz< 1) then
       call release (tbl)
       RETURN
    end if


    ! Finding bad<>good changes. (Better do this with linked list)

    istart=0
    previous = .false.
    do i=1, size(good)
       !write(*,*) 'i previous goodi =', i, previous, good(i)
       if (.not. previous .and. good(i))  istart=i  ! bad -> good
 
       if (previous .and. .not. good(i)) then  ! good -> bad
          istop = i-1
          !write(*,*) 'Adding GTI from istart istop =', istart, istop 
          call addGTI(tbl, fromtime= time(istart) - 0.5*exposureTime(istart), &
                             totime= time(istop) + 0.5*exposureTime(istop))
       end if

       previous = good(i)
    end do

    ! last frame
    if (istart/=0 .and. good(sz)) then
       !write(*,*) 'Adding GTI from istart istop =', istart, sz 
       call addGTI(tbl, fromtime= time(istart) - 0.5*exposureTime(istart), &
                          totime= time(sz) + 0.5*exposureTime(sz))
    end if
  end function createGTI
end module gti_mod
